@@ -1,53 +1,46 @@
-/*============================================================================
-  CMake - Cross Platform Makefile Generator
-  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
-
-  Distributed under the OSI-approved BSD License (the "License");
-  see accompanying file Copyright.txt for details.
-
-  This software is distributed WITHOUT ANY WARRANTY; without even the
-  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-  See the License for more information.
-============================================================================*/
+/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
+   file Copyright.txt or https://cmake.org/licensing for details.  */
 #include "cmInstallExportGenerator.h"
 
-#include <stdio.h>
-
-#include "cmake.h"
-#include "cmInstallTargetGenerator.h"
-#include "cmGeneratedFileStream.h"
-#include "cmMakefile.h"
-#include "cmLocalGenerator.h"
-#include "cmGlobalGenerator.h"
-
-#include "cmInstallFilesGenerator.h"
+#include <algorithm>
+#include <map>
+#include <sstream>
+#include <utility>
 
+#ifdef CMAKE_BUILD_WITH_CMAKE
+#include "cmExportInstallAndroidMKGenerator.h"
+#endif
 #include "cmExportInstallFileGenerator.h"
 #include "cmExportSet.h"
+#include "cmInstallType.h"
+#include "cmLocalGenerator.h"
+#include "cmSystemTools.h"
+#include "cmake.h"
 
-//----------------------------------------------------------------------------
 cmInstallExportGenerator::cmInstallExportGenerator(
-  cmExportSet* exportSet,
-  const char* destination,
-  const char* file_permissions,
-  std::vector<std::string> const& configurations,
-  const char* component,
-  MessageLevel message,
-  const char* filename, const char* name_space,
-  bool exportOld)
-  :cmInstallGenerator(destination, configurations, component, message)
-  ,ExportSet(exportSet)
-  ,FilePermissions(file_permissions)
-  ,FileName(filename)
-  ,Namespace(name_space)
-  ,ExportOld(exportOld)
-  ,LocalGenerator(0)
+  cmExportSet* exportSet, const char* destination,
+  const char* file_permissions, std::vector<std::string> const& configurations,
+  const char* component, MessageLevel message, bool exclude_from_all,
+  const char* filename, const char* name_space, bool exportOld, bool android)
+  : cmInstallGenerator(destination, configurations, component, message,
+                       exclude_from_all)
+  , ExportSet(exportSet)
+  , FilePermissions(file_permissions)
+  , FileName(filename)
+  , Namespace(name_space)
+  , ExportOld(exportOld)
+  , LocalGenerator(CM_NULLPTR)
 {
-  this->EFGen = new cmExportInstallFileGenerator(this);
+  if (android) {
+#ifdef CMAKE_BUILD_WITH_CMAKE
+    this->EFGen = new cmExportInstallAndroidMKGenerator(this);
+#endif
+  } else {
+    this->EFGen = new cmExportInstallFileGenerator(this);
+  }
   exportSet->AddInstallation(this);
 }
 
-//----------------------------------------------------------------------------
 cmInstallExportGenerator::~cmInstallExportGenerator()
 {
   delete this->EFGen;
@@ -59,23 +52,17 @@ void cmInstallExportGenerator::Compute(cmLocalGenerator* lg)
   this->ExportSet->Compute(lg);
 }
 
-//----------------------------------------------------------------------------
 void cmInstallExportGenerator::ComputeTempDir()
 {
   // Choose a temporary directory in which to generate the import
   // files to be installed.
-  this->TempDir =
-      this->LocalGenerator->GetCurrentBinaryDirectory();
+  this->TempDir = this->LocalGenerator->GetCurrentBinaryDirectory();
   this->TempDir += cmake::GetCMakeFilesDirectory();
   this->TempDir += "/Export";
-  if(this->Destination.empty())
-    {
+  if (this->Destination.empty()) {
     return;
-    }
-  else
-    {
-    this->TempDir += "/";
-    }
+  }
+  this->TempDir += "/";
 
   // Enforce a maximum length.
   bool useMD5 = false;
@@ -84,55 +71,67 @@ void cmInstallExportGenerator::ComputeTempDir()
 #else
   std::string::size_type const max_total_len = 1000;
 #endif
-  if(this->TempDir.size() < max_total_len)
-    {
+  // Will generate files of the form "<temp-dir>/<base>-<config>.<ext>".
+  std::string::size_type const len = this->TempDir.size() + 1 +
+    this->FileName.size() + 1 + this->GetMaxConfigLength();
+  if (len < max_total_len) {
     // Keep the total path length below the limit.
-    std::string::size_type max_len = max_total_len - this->TempDir.size();
-    if(this->Destination.size() > max_len)
-      {
+    std::string::size_type const max_len = max_total_len - len;
+    if (this->Destination.size() > max_len) {
       useMD5 = true;
-      }
     }
-  else
-    {
+  } else {
     useMD5 = true;
-    }
-  if(useMD5)
-    {
+  }
+  if (useMD5) {
     // Replace the destination path with a hash to keep it short.
-    this->TempDir +=
-      cmSystemTools::ComputeStringMD5(this->Destination);
-    }
-  else
-    {
+    this->TempDir += cmSystemTools::ComputeStringMD5(this->Destination);
+  } else {
     std::string dest = this->Destination;
     // Avoid unix full paths.
-    if(dest[0] == '/')
-      {
+    if (dest[0] == '/') {
       dest[0] = '_';
-      }
+    }
     // Avoid windows full paths by removing colons.
-    cmSystemTools::ReplaceString(dest, ":", "_");
+    std::replace(dest.begin(), dest.end(), ':', '_');
     // Avoid relative paths that go up the tree.
     cmSystemTools::ReplaceString(dest, "../", "__/");
     // Avoid spaces.
-    cmSystemTools::ReplaceString(dest, " ", "_");
+    std::replace(dest.begin(), dest.end(), ' ', '_');
     this->TempDir += dest;
+  }
+}
+
+size_t cmInstallExportGenerator::GetMaxConfigLength() const
+{
+  // Always use at least 8 for "noconfig".
+  size_t len = 8;
+  if (this->ConfigurationTypes->empty()) {
+    if (this->ConfigurationName.size() > 8) {
+      len = this->ConfigurationName.size();
     }
+  } else {
+    for (std::vector<std::string>::const_iterator ci =
+           this->ConfigurationTypes->begin();
+         ci != this->ConfigurationTypes->end(); ++ci) {
+      if (ci->size() > len) {
+        len = ci->size();
+      }
+    }
+  }
+  return len;
 }
 
-//----------------------------------------------------------------------------
 void cmInstallExportGenerator::GenerateScript(std::ostream& os)
 {
   // Skip empty sets.
-  if(ExportSet->GetTargetExports()->empty())
-    {
+  if (ExportSet->GetTargetExports()->empty()) {
     std::ostringstream e;
-    e << "INSTALL(EXPORT) given unknown export \""
-      << ExportSet->GetName() << "\"";
+    e << "INSTALL(EXPORT) given unknown export \"" << ExportSet->GetName()
+      << "\"";
     cmSystemTools::Error(e.str().c_str());
     return;
-    }
+  }
 
   // Create the temporary directory in which to store the files.
   this->ComputeTempDir();
@@ -147,36 +146,27 @@ void cmInstallExportGenerator::GenerateScript(std::ostream& os)
   this->EFGen->SetExportFile(this->MainImportFile.c_str());
   this->EFGen->SetNamespace(this->Namespace);
   this->EFGen->SetExportOld(this->ExportOld);
-  if(this->ConfigurationTypes->empty())
-    {
-    if(!this->ConfigurationName.empty())
-      {
+  if (this->ConfigurationTypes->empty()) {
+    if (!this->ConfigurationName.empty()) {
       this->EFGen->AddConfiguration(this->ConfigurationName);
-      }
-    else
-      {
+    } else {
       this->EFGen->AddConfiguration("");
-      }
     }
-  else
-    {
-    for(std::vector<std::string>::const_iterator
-          ci = this->ConfigurationTypes->begin();
-        ci != this->ConfigurationTypes->end(); ++ci)
-      {
+  } else {
+    for (std::vector<std::string>::const_iterator ci =
+           this->ConfigurationTypes->begin();
+         ci != this->ConfigurationTypes->end(); ++ci) {
       this->EFGen->AddConfiguration(*ci);
-      }
     }
+  }
   this->EFGen->GenerateImportFile();
 
   // Perform the main install script generation.
   this->cmInstallGenerator::GenerateScript(os);
 }
 
-//----------------------------------------------------------------------------
-void
-cmInstallExportGenerator::GenerateScriptConfigs(std::ostream& os,
-                                                Indent const& indent)
+void cmInstallExportGenerator::GenerateScriptConfigs(std::ostream& os,
+                                                     Indent indent)
 {
   // Create the main install rules first.
   this->cmInstallGenerator::GenerateScriptConfigs(os, indent);
@@ -184,25 +174,22 @@ cmInstallExportGenerator::GenerateScriptConfigs(std::ostream& os,
   // Now create a configuration-specific install rule for the import
   // file of each configuration.
   std::vector<std::string> files;
-  for(std::map<std::string, std::string>::const_iterator
-        i = this->EFGen->GetConfigImportFiles().begin();
-      i != this->EFGen->GetConfigImportFiles().end(); ++i)
-    {
+  for (std::map<std::string, std::string>::const_iterator i =
+         this->EFGen->GetConfigImportFiles().begin();
+       i != this->EFGen->GetConfigImportFiles().end(); ++i) {
     files.push_back(i->second);
     std::string config_test = this->CreateConfigTest(i->first);
     os << indent << "if(" << config_test << ")\n";
-    this->AddInstallRule(os, this->Destination,
-                         cmInstallType_FILES, files, false,
-                         this->FilePermissions.c_str(), 0, 0, 0,
-                         indent.Next());
+    this->AddInstallRule(os, this->Destination, cmInstallType_FILES, files,
+                         false, this->FilePermissions.c_str(), CM_NULLPTR,
+                         CM_NULLPTR, CM_NULLPTR, indent.Next());
     os << indent << "endif()\n";
     files.clear();
-    }
+  }
 }
 
-//----------------------------------------------------------------------------
 void cmInstallExportGenerator::GenerateScriptActions(std::ostream& os,
-                                                     Indent const& indent)
+                                                     Indent indent)
 {
   // Remove old per-configuration export files if the main changes.
   std::string installedDir = "$ENV{DESTDIR}";
@@ -214,6 +201,7 @@ void cmInstallExportGenerator::GenerateScriptActions(std::ostream& os,
   Indent indentN = indent.Next();
   Indent indentNN = indentN.Next();
   Indent indentNNN = indentNN.Next();
+  /* clang-format off */
   os << indentN << "file(DIFFERENT EXPORT_FILE_CHANGED FILES\n"
      << indentN << "     \"" << installedFile << "\"\n"
      << indentN << "     \"" << this->MainImportFile << "\")\n";
@@ -227,11 +215,12 @@ void cmInstallExportGenerator::GenerateScriptActions(std::ostream& os,
   os << indentNN << "endif()\n";
   os << indentN << "endif()\n";
   os << indent << "endif()\n";
+  /* clang-format on */
 
   // Install the main export file.
   std::vector<std::string> files;
   files.push_back(this->MainImportFile);
-  this->AddInstallRule(os, this->Destination,
-                       cmInstallType_FILES, files, false,
-                       this->FilePermissions.c_str(), 0, 0, 0, indent);
+  this->AddInstallRule(os, this->Destination, cmInstallType_FILES, files,
+                       false, this->FilePermissions.c_str(), CM_NULLPTR,
+                       CM_NULLPTR, CM_NULLPTR, indent);
 }
